#filterResponse.py
import numpy as np,scipy.signal as signal
import matplotlib.pyplot as plt

def butterFilter(lowCut, highCut, sampleRate, order, filterType):
    semiSampleRate = sampleRate*0.5
    low,high = lowCut/semiSampleRate, highCut/semiSampleRate
    b,a = signal.butter(order,[low,high],btype=filterType)
    print(f"{filterType} filter, order = {order}")
    print("b.shape:",b.shape,"a.shape:",a.shape)
    print("b=",b, "\na=",a)
    return b,a

sampleRate = 2000
plt.figure(figsize=(12,5))
ax0 = plt.subplot(121)
for k in [2, 3, 4]:
    b, a = butterFilter(3,70,sampleRate, k,"bandpass")
    w, h = signal.freqz(b, a, worN=2000)
    ax0.plot(w*sampleRate*0.5/np.pi,np.abs(h),label=f"order {k}")

ax1 = plt.subplot(122)
for k in [2, 3, 4]:
    b, a = butterFilter(48, 52,sampleRate,k,"bandstop")
    w, h = signal.freqz(b, a, worN=2000)
    ax1.plot(w*sampleRate*0.5/np.pi,np.abs(h),label=f"order {k}")
ax1.set_xlim(35,65)

for a in [ax0,ax1]:
    a.plot([0, 0.5 * sampleRate], [np.sqrt(0.5), np.sqrt(0.5)],
             '--', label='sqrt(0.5)')
    a.set_xlabel('Frequency (Hz)')
    a.set_ylabel('Gain')
    a.grid(True)
    a.legend(loc='best')

plt.subplots_adjust(0.05,0.05,0.95,0.95,wspace=0.2)
plt.show()
