#iirphase.py
import numpy as np, scipy.signal as signal 
from matplotlib import pyplot as plt

def butterFilter(lowCut, highCut, sampleRate, order, filterType):
    semiSampleRate = sampleRate*0.5
    low,high = lowCut/semiSampleRate, highCut/semiSampleRate
    b,a = signal.butter(order,[low,high],btype=filterType)
    return b,a

sampleRate = 2000					#采样频率,每秒2000个样本
x = np.fromfile("ecgsignal.dat",dtype=np.float32)
x -= np.mean(x)                     #去直流分量

b,a = butterFilter(3,70,sampleRate,4,"bandpass")   
y1 = signal.lfilter(b,a,x)           #进行有相移滤波
y2 = signal.filtfilt(b,a,x)          #进行无相移滤波
b,a = butterFilter(48,52,sampleRate,2,"bandstop")
y1 = signal.lfilter(b,a,y1)           #进行有相移滤波
y2 = signal.filtfilt(b,a,y2)          #进行无相移滤波

sampleCount = x.shape[0]			#采样数
t = np.linspace(0,sampleCount/sampleRate,sampleCount)
xFFT = np.abs(np.fft.rfft(x)/sampleCount)  #快速傅里叶变换计算频谱
xFreqs = np.linspace(0, sampleRate/2, sampleCount//2+1)

plt.figure(figsize=(10,6))
ax0 = plt.subplot(311)             #画有相移信号
ax0.set_xlabel("Time(s) - Original Signal")
ax0.set_ylabel("Amp(μV)")
ax0.axis([np.min(t),np.max(t),-2000,2000])
ax0.plot(t,x)

ax1 = plt.subplot(312)             #画有相移信号
ax1.set_xlabel("Time(s) - With Phase Shift")
ax1.set_ylabel("Amp(μV)")
ax1.axis([np.min(t),np.max(t),-2000,2000])
ax1.plot(t,y1)

ax2 = plt.subplot(313)             #画无相移信号
ax2.set_xlabel("Time(s) - Without Phase Shift")
ax2.set_ylabel("Amp(μV)")
ax2.axis([np.min(t),np.max(t),-2000,2000])
ax2.plot(t,y2)

plt.subplots_adjust(0.1,0.1,0.95,0.95,hspace=0.5)
plt.show()

